﻿namespace FaceDetection
{
  using System;
  using System.Collections.Generic;
  using System.IO;
  using System.Linq;
  using System.Xml;
  using FaceDetection.Convolution;
  using SharpNeat.Genomes.Neat;
  using SharpNeat.Phenomes;

  class Booster
  {
    public static void WriteScoutingMatrix()
    {
      var factories = Common.GetFactories();
      var decoders = Common.GetDecoders();

      var boxes = Directory.EnumerateFiles("Log", "*.xml")
        .Select(path => Common.LoadNetwork(path))
        .ToList();

      var evaluator = new Evaluator(new Loader(), 0);
      using (var writer = new StreamWriter("ScoutingMatrix.txt"))
      {
        foreach (var row in evaluator.CreateScoutingMatrix(boxes))
        {
          writer.WriteLine(row);
        }
      }
    }

    private const int TotalNetworks = 50;

    public static void Boost()
    {
      var scouting = new List<byte[]>();
      Console.WriteLine("Reading matrix...");
      using (var reader = new StreamReader("ScoutingMatrix.txt"))
      {
        while (true)
        {
          var line = reader.ReadLine();
          if (String.IsNullOrEmpty(line))
          {
            break;
          }
          scouting.Add(line.Select(x => x == '0' ? (byte)0 : (byte)1).ToArray());
        }
      }
      scouting.TrimExcess();

      var indices = new List<int>();
      var coeffs = new List<double>();
      var outputs = new double[scouting.Count];

      int positivesCount = Loader.PositivesCount();
      double[] weights = Enumerable.Repeat(1.0, scouting.Count).ToArray();

      for (int k = 0; k < TotalNetworks; ++k)
      {
        double minError = Double.MaxValue;
        int minIndex = -1;
        for (int i = 0; i < scouting[0].Length; ++i)
        {
          double error = 0;
          for (int j = 0; j < scouting.Count; ++j)
          {
            error += scouting[j][i] * weights[j];
          }

          if (error < minError)
          {
            minIndex = i;
            minError = error;
          }
        }

        indices.Add(minIndex);

        double em = minError / weights.Sum();
        coeffs.Add(Math.Log((1 - em) / em) / 2);

        for (int j = 0; j < scouting.Count; ++j)
        {
          if (scouting[j][minIndex] == 1)
          {
            weights[j] *= Math.Sqrt((1 - em) / em);
          }
          else
          {
            weights[j] *= Math.Sqrt(em / (1 - em));
          }
        }

        int positivesCorrect = 0;
        int negativesCorrect = 0;
        for (int i = 0; i < positivesCount; ++i)
        {
          double networkWeight = coeffs.Last();
          int networkOutput = scouting[i][minIndex] == 1 ? -1 : 1;
          outputs[i] += networkOutput * networkWeight;
          if (outputs[i] > 0)
          {
            ++positivesCorrect;
          }
        }

        for (int i = positivesCount; i < scouting.Count; ++i)
        {
          double networkWeight = coeffs.Last();
          int networkOutput = scouting[i][minIndex] == 1 ? 1 : -1;
          outputs[i] += networkOutput * networkWeight;
          if (outputs[i] <= 0)
          {
            ++negativesCorrect;
          }
        }

        Console.WriteLine("{0,3} : {1,8:0.00000} | {2,8:0.00000} : {3,8:0.00000} | {4,8:0.00000}",
          indices.Last(), coeffs.Last(), 100.0 * positivesCorrect / positivesCount,
          100.0 * negativesCorrect / (scouting.Count - positivesCount),
          100.0 * (positivesCorrect + negativesCorrect) / scouting.Count);
      }
    }
  }
}
