﻿namespace FaceDetection.Convolution
{
  using System.Collections.Generic;
  using System.Linq;
  using SharpNeat.Genomes.Neat;
  using SharpNeat.Network;
  
  public class  ConvGenomeFactory : NeatGenomeFactory
  {
    public ConvGenomeFactory(int inputNeuronCount, int outputNeuronCount,
      IActivationFunctionLibrary activationFnLibrary, NeatGenomeParameters neatGenomeParams)
      : base(inputNeuronCount, outputNeuronCount, activationFnLibrary, neatGenomeParams)
    {
    }

    public override NeuronGene CreateNeuronGene(uint innovationId, NodeType neuronType)
    {
      if (neuronType == NodeType.Bias)
      {
        var func = _activationFnLibrary.GetFunctionList()[0];
        return new NeuronGene(innovationId, neuronType, func.Id);
      }
      else if (neuronType == NodeType.Input)
      {
        var func = ActivationFnLibrary.GetFunctionList().Last();
        double[] auxArgs = func.ActivationFunction.GetRandomAuxArgs(Rng, NeatGenomeParameters.ConnectionWeightRange);
        return new NeuronGene(innovationId, neuronType, func.Id, auxArgs);
      }
      else
      {
        var info = ActivationFnLibrary.GetRandomFunction(Rng);
        var func = info.ActivationFunction;
        double[] auxArgs = null;
        if (func.AcceptsAuxArgs)
        {
          auxArgs = func.GetRandomAuxArgs(Rng, NeatGenomeParameters.ConnectionWeightRange);
        }
        return new NeuronGene(innovationId, neuronType, info.Id, auxArgs);
      }
    }

    public static IActivationFunctionLibrary CreateLibraryConv(IActivationFunctionLibrary library, int inputs)
    {
      var list = new List<ActivationFunctionInfo>(library.GetFunctionList());
      int id = list.Select(info => info.Id).Max() + 1;
      list.Add(new ActivationFunctionInfo(id, 0, new ConvFunction(inputs)));
      return new DefaultActivationFunctionLibrary(list);
    }
  }
}
