﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace FaceDetection
{
  class Statistics
  {
    public static void CalculateProgress()
    {
      double[,] bestFitness = new double[12, 40];
      double[,] avgFitness = new double[12, 40];
      double[,] posFitness = new double[12, 40];
      double[,] negFitness = new double[12, 40];
      double[,] bestComplexity = new double[12, 40];
      double[,] avgComplexity = new double[12, 40];
      foreach (var path in Directory.EnumerateFiles("Log", "*.txt"))
      {
        string strIndex = Path.GetFileNameWithoutExtension(path).Split('_')[0].TrimStart('0');
        int index = strIndex.Length != 0 ? Int32.Parse(strIndex) : 0;

        using (var reader = new StreamReader(path))
        {
          reader.ReadLine();
          for (int i = 0; i < 40; ++i)
          {
            string[] parts = reader.ReadLine().Split('\t');
            bestFitness[index, i] += Double.Parse(parts[4]);
            avgFitness[index, i] += Double.Parse(parts[2]);
            posFitness[index, i] += Double.Parse(parts[6]);
            negFitness[index, i] += Double.Parse(parts[7]);
            bestComplexity[index, i] += Double.Parse(parts[5]);
            avgComplexity[index, i] += Double.Parse(parts[3]);

            for (int j = 0; j < 24; ++j)
            {
              reader.ReadLine();
            }
          }
        }
      }

      for (int k = 0; k < 12; ++k)
      {
        using (var writer = new StreamWriter(String.Format(@"D:\{0:00}.txt", k)))
        {
          for (int i = 0; i < 40; ++i)
          {
            writer.WriteLine("{0:0.00000}\t{1:0.00000}\t{2:0.00000}\t{3:0.00000}\t{4:0.00000}\t{5:0.00000}",
              bestFitness[k, i] / 30, avgFitness[k, i] / 30, posFitness[k, i] / 30, negFitness[k, i] / 30, bestComplexity[k, i] / 30, avgComplexity[k, i] / 30);
          }
        }
      }
    }

    public static void FindBest()
    {
      double[] bestFitness = new double[12];
      string[] bestName = new string[12];

      foreach (var path in Directory.EnumerateFiles("Log", "*.txt"))
      {
        string strIndex = Path.GetFileNameWithoutExtension(path).Split('_')[0].TrimStart('0');
        int index = strIndex.Length != 0 ? Int32.Parse(strIndex) : 0;

        string lastLine = null;
        using (var reader = new StreamReader(path))
        {
          string line = reader.ReadLine();
          if (String.IsNullOrEmpty(line))
          {
            break;
          }
          lastLine = line;
        }

        double fitness = Double.Parse(lastLine.Split('\t')[4]);
        if (fitness > bestFitness[index])
        {
          bestFitness[index] = fitness;
          bestName[index] = Path.GetFileNameWithoutExtension(path);
        }
      }

      for (int i = 0; i < 12; ++i)
      {
        Console.WriteLine(bestName[i] + " " + bestFitness[i]);
      }
    }

    public static void CalculateBest()
    {
      var best = new[] { "00_0025", "01_0017", "02_0004", "03_0020", "04_0024", "05_0023", "06_0004", "07_0020", "08_0024", "09_0002", "10_0026", "11_0017" };
      var factories = Common.GetFactories();
      var decoders = Common.GetDecoders();

      var boxes = best.Select(b => Path.Combine("Log", b + ".xml"))
        .Select(path => Common.LoadNetwork(path))
        .ToList();

      var evaluator = new Evaluator(new Loader(), 0);
      var matrix = evaluator.CreateScoutingMatrix(boxes).ToList();

      for (int i = 0; i < 12; ++i)
      {
        int fp = 0;
        for (int j = 0; j < 1994; ++j)
        {
          if (matrix[j][i] == '1')
          {
            ++fp;
          }
        }

        int fn = 0;
        for (int j = 1994; j < matrix.Count; ++j)
        {
          if (matrix[j][i] == '1')
          {
            ++fn;
          }
        }

        Console.WriteLine("{0} {1}", fp, fn);
      }
    }
  }
}
